@@ -4,6 +4,7 @@
 #  gbottler.py - a SSTP Bottle Client
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
+#  Copyright (C) 2004 by Takuya KAWAHARA <num@sann.ne.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -12,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: gbottler.py,v 1.109 2004/06/28 06:45:47 atzm Exp $
+# $Id: gbottler.py,v 1.115 2004/07/27 20:04:43 atzm Exp $
 #
 
 import os
@@ -37,6 +38,7 @@ from downloader   import Downloader
 from queryluid    import LUIDInfo
 from bottleparser import *
 from gvwindow     import *
+from svgmanager   import *
 
 from common import *
 
@@ -344,6 +346,8 @@ class Application:
 		self.gvwindow = GVWindow(os.path.join(open_bottlecase(), 'gviewer', 'Ghost'),
 								 os.path.join(open_bottlecase(), 'gviewer', 'ghost.txt'))
 
+		self.svgmanager = SVGManager(self, os.path.join(open_bottlecase(), 'gviewer'))
+
 		self.menu_list["/%s/%s" % (self.msg["window"][1],
 								   self.msg["logs"])].connect("activate", self.log_window.open, None)
 
@@ -353,12 +357,18 @@ class Application:
 		self.menu_list["/%s/%s" % (self.msg["window"][1],
 								   self.msg["gviewer"])].connect("activate", self.gvwindow.open, None)
 
+		self.menu_list["/%s/%s" % (self.msg["window"][1],
+								   self.msg["svgmanager"])].connect("activate", self.svgmanager.open, None)
+
 		self.execute_command = 'GetNames'
 		self.target_ghost = ""
 		self.sakuraname = ""
 
 		self.local_ghost = []
 		self.local_ghost_last_notified = 0
+
+		self.fmo_ghosts = self.gvwindow.get_fmo_ghosts()
+
 		self.send_ghost_names()
 
 		self.c_e_id = 1 # channel label eventbox handler id
@@ -460,6 +470,12 @@ class Application:
 									   prefs['sync'], prefs['error'], prefs['URL'])
 		self.textmanager.coloring()
 
+	def notify_svg_changed(self):
+		self.gvwindow.notify_svg_changed()
+		self.fmo_ghosts = self.gvwindow.get_fmo_ghosts()
+		if self.client.connected:
+			self.rebuild_ghostmenu()
+
  	def popup(self, widget=None, event=None):
  		if event.button == 3:
  			self.popup_menu.popup(None, widget, None, event.button, 0)
@@ -557,6 +573,13 @@ class Application:
 		self.monitor_clear()
 		self.monitor_insert(name + unicode(_('channel was closed.'), 'utf-8'))
 
+	def uniq_ghosts(self):
+		ghosts = self.local_ghost + self.fmo_ghosts
+		for ghostname in ghosts:
+			while ghosts.count(ghostname) - 1:
+				ghosts.remove(ghostname)
+		return ghosts
+
 	def rebuild_ghostmenu(self):
 		ghost_item = self.menu_list["/%s" % self.msg["ghost"][1]]
 		ghost_item.deselect()
@@ -566,7 +589,7 @@ class Application:
 			self.g_e_box.disconnect(self.g_e_id)
 
 		current = self.target_ghost
-		ghosts = [self.DEFAULT_TARGET] + self.local_ghost
+		ghosts = [self.DEFAULT_TARGET] + self.uniq_ghosts() #self.local_ghost + self.fmo_ghosts
 		name = ""
 		for n in range(len(self.client.channels)):
 			name = self.client.channels[n]["ghost"]
@@ -1267,6 +1290,7 @@ class Application:
 			"logs":    unicode(_("Logs"), "utf-8"),
 			"phrases": unicode(_("Phrases"), "utf-8"),
 			"gviewer": unicode(_("GViewer"), "utf-8"),
+			"svgmanager": unicode(_("SVGManager"), "utf-8"),
 			"help":   [unicode(_("Help(_H)"), "utf-8"),
 					   unicode(_("Help(H)"), "utf-8")],
 			"about":  unicode(_("About"), "utf-8"),
@@ -1299,6 +1323,7 @@ class Application:
 			( "/%s/%s" % (self.msg["window"][1], self.msg["logs"]), "<control>L", None, 0, None ),
 			( "/%s/%s" % (self.msg["window"][1], self.msg["phrases"]), None, None, 0, None ),
 			( "/%s/%s" % (self.msg["window"][1], self.msg["gviewer"]), None, None, 0, None ),
+			( "/%s/%s" % (self.msg["window"][1], self.msg["svgmanager"]), None, None, 0, None ),
 
 			( "/%s" % self.msg["help"][0], None, None, 0, "<LastBranch>" ),
 			( "/%s/%s" % (self.msg["help"][1], self.msg["about"]), None, self.about, 0, None ),
